* {
	font-family: Arial, Helvetica, sans-serif;
	line-height: 1.2;
}

body {
	width: 800px;
}

a {
	color: #1144aa;
	text-decoration: none;
}

a:hover {
	text-decoration: underline;
}

p, pre, h4 {
	margin-left: 1em;
}

h2 {
	margin: 0;
}

h3 {
	color: #444;
	border-left: 6px solid #777;
	border-bottom: 1px solid #777;
	padding-left: 0.5em;
	padding-bottom: 0.3em;
	padding-top: 0.2em;
	margin-top: 1.5em;
	margin-bottom: 0;
}

h4 {
	margin-bottom: 0;
}

pre, tt, code {
	font-family: "Courier New", Courier, monospace;
}

pre {
	background-color: #f0f0f0;
	padding: 0.5em;
}

.version {
	font-size: 0.8em;
}

ul {
	margin-bottom: 0.5em;
}

ul li {
	margin-top: 0.2em;
	margin-bottom: 0.2em;
}

ul li ul li {
	font-size: 0.9em;
	margin-top: 0.2em;
	margin-bottom: 0.2em;
}

ul.versionlist li {
	margin-bottom: 1em;
}

ul.versionlist li div.note {
	font-size: 0.9em;
	margin-top: 0.5em;
}

ul.versionlist li.oldversion {
	color: #777;
	font-size: 0.8em;
}

.hashes {
	font-size: 0.8em;
}

.hashes tt {
	font-size: 1.1em;
}
